@@ -55,4 +55,5 @@
 #' @importFrom utils installed.packages
 #' @docType package
 #' @name multiblock
-NULL
\ No newline at end of file
+#' @keywords internal
+"_PACKAGE"
